#!/usr/bin/perl -w

# Copyright 1999, 2000, 2001 (c) Thomas Erskine <thomas.erskine@sourceworks.com>
# See the COPYRIGHT file with the distribution.

# graph.cgi - dynamically emit a remstats graph, for static pages
# $Id: graph.cgi.in,v 1.5 2001/08/28 15:22:24 remstats Exp $

# - - -   Configuration   - - -

use strict;

# What is this program called, for file-names and error-messages.
$main::prog = 'graph.cgi';
# So -T will stop whinging.
$ENV{PATH} = '/usr/local/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/usr/lib/remstats/bin';
# Where is the config dir
$main::config_dir = '/etc/remstats/config';
# Debugging anyone?
$main::debug = 0;
# Image only, i.e. no headers?
$main::image_only = 0;

# - - -   Version History   - - -

(undef, $main::version) = split(' ', '$Revision: 1.5 $');

# - - -   Setup   - - -

use lib '.', '/usr/lib/remstats/lib', '/usr/lib/perl5/';
use Getopt::Std;
use RRDs;
require "remstats.pl";
require "cgistuff.pl";
require "htmlstuff.pl";

getopts("d:f:hi");

if (defined $main::opt_h) { &usage; } # no return
if (defined $main::opt_f) { $main::config_dir = $main::opt_f; }
if (defined $main::opt_d) { $main::debug = $main::opt_d; }
if (defined $main::opt_i) { $main::image_only = 1; }

&read_config_dir($main::config_dir, 'general', 'html', 'links', 'colors',
	'tools', 'times', 'oids', 'rrds', 'customgraphs', 'groups', 
	'host-templates', 'hosts', 'view-templates', 'views');

# Initialize some useful variables
%main::R = &cgi_request;
&cgi_sendheaders("Description: Remstats Graph Generator $main::prog version $main::version",
	'Content-type: image/png') unless($main::image_only);
$main::url = 'http://' . &cgi_var("SERVER_NAME") . ':' . 
	&cgi_var('SERVER_PORT');
my $script_name = &cgi_var('SCRIPT_NAME');
if ($script_name =~ m#^/#) { $main::url .= $script_name; }
else { $main::url .= '/'. $script_name; }

$| = 1; # no buffering please
print cgi_fmtrequest(%main::R) if ($main::debug);

my ($host, $realrrd, $wildrrd, $wildpart, $fixedrrd, $graph, $graphtime,
	$customgraph);
if (defined $main::R{'host'}) {
	if ($main::R{'host'} =~ /^([-.a-zA-Z0-9]+)$/) {
		$host = lc $1;
		unless (defined $main::config{HOST}{$host}) {
			&abort("unknown host '$host'");
		}
		&debug("  host='$host'") if ($main::debug);
	}
	else { &abort("bad host spec '$main::R{'host'}'"); }
}
elsif (defined $main::R{'customgraph'}) {
	if ($main::R{'customgraph'} =~ /^([-a-zA-Z0-9._]+)$/) {
		$customgraph = $1;
		unless (defined $main::config{CUSTOMGRAPH}{$customgraph}) {
			&abort("unknown customgraph '$customgraph'") if ($main::debug);
		}
		&debug("  customgraph='$customgraph'") if ($main::debug);
	}
	else { &abort("bad customgraph spec '$main::R{'customgraph'}'"); }
}
else { &abort("missing host or customgraph"); }

if (defined $host and defined $main::R{'rrd'}) {
	if ($main::R{'rrd'} =~ /^([-:\/.a-zA-Z0-9]+)$/) {
		$realrrd = lc $1;
		($wildrrd, $wildpart, undef, $fixedrrd) = &get_rrd($realrrd);
		if (!defined $wildrrd) {
			&abort("unknown rrd $realrrd");
		}
		&debug(" rrd='$realrrd', wild='$wildrrd'") if ($main::debug);
	}
	else { &abort("bad rrd spec '$main::R{'rrd'}'"); }
}
elsif (defined $customgraph) { }
else { &abort("missing rrd"); }
0;

if (defined $host and defined $main::R{'graph'}) {
	if ($main::R{'graph'} =~ /^([-.a-zA-Z0-9\*]+)$/) {
		$graph = lc $1;
		unless (defined $main::config{RRD}{$wildrrd}{GRAPH}{$graph}) {
			&abort("unknown graph '$graph' for rrd '$realrrd'");
		}
		&debug("  graph='$graph'") if ($main::debug);
	}
	else { &abort("bad graph spec '$main::R{'graph'}'"); }
}
elsif (defined $customgraph) { }
else { &abort("missing graph"); }

if (defined $main::R{'time'}) { $main::R{'graphtime'} = $main::R{'time'}; }
if (defined $main::R{'graphtime'}) {
	if ($main::R{'graphtime'} =~ /^([-a-zA-Z0-9]+)$/) {
		$graphtime = lc $1;
		unless (defined $main::config{TIME}{$graphtime}) {
			&abort("unknown timespan '$graphtime'"); 
		}
		&debug("  time='$graphtime'") if ($main::debug);
	}
	else { &abort("bad graphtime spec '$main::R{'graphtime'}'"); }
}
else { &abort("missing graphtime"); }

# Prepare the graph start/end and image-type
my @rrdgraph = ('--start', $main::config{TIME}{$graphtime}{START});
unless ($main::config{TIME}{$graphtime}{FINISH} == 0) {
	push @rrdgraph, '--end', $main::config{TIME}{$graphtime}{FINISH};
}
push @rrdgraph, '--imgformat', $main::config{IMAGETYPE};

# Host-specific graph
if (defined $host) {
	my $rrdfile = $main::config{DATADIR} .'/'. $host .'/'. $fixedrrd .'.rrd';
	push @rrdgraph, &do_subs($rrdfile, $realrrd, $wildpart, $host, undef, 
		$graph, $graphtime, @{$main::config{RRD}{$wildrrd}{GRAPH}{$graph}});
}

# Customgraph
elsif (defined $customgraph) {
	push @rrdgraph, &do_subs( undef, undef, undef, undef, undef, undef,
		$graphtime, @{$main::config{CUSTOMGRAPH}{$customgraph}{GRAPH}});
}
else { &abort("neither host nor customgraph are set"); }

# Now make a graph
my $graphfile = ($main::debug) ? '/tmp/graph.png' : '-';
RRDs::graph $graphfile, @rrdgraph;
my $error;
if ($error = RRDs::error) { &abort('RRD error: '.$error); }

exit 0;

#----------------------------------------------------------------- abort ---
sub abort {
	my ($msg) = @_;
	&error("ABORT: $msg");
	exit 1;
}

#------------------------------------------------- debug ---
sub debug {
	my ($msg) = @_;

	&error("DEBUG: $!");
}

#-------------------------------------------------- error ---
sub error {
	my ($msg) = @_;

	open (ERROR, ">>/tmp/errors") or 
		die "$main::prog: can't open /tmp/errors: $!\n";
	print ERROR "ERROR: $msg\n";
	close (ERROR);
}

#---------------------------------------------------- usage ---
sub usage {
	print STDERR <<"EOD_USAGE";
$main::prog version $main::version
usage: $main::prog [options]
where options are:
	-d ddd  set debugging level to 'ddd'
	-f fff  use 'fff' for config-dir [$main::config_dir]
	-h      show this help
EOD_USAGE
	exit 0;
}

#----------------------------------------------- keep_strict_happy ---
sub keep_strict_happy {
	$main::opt_h = $main::opt_i = 0;
}
