#!/usr/bin/perl -Tw

# Copyright 1999, 2000, 2001 (c) Thomas Erskine <thomas.erskine@sourceworks.com>
# See the COPYRIGHT file with the distribution.

# whois.cgi - let people get whois info
# $Id: whois.cgi.in,v 1.5 2001/08/28 15:22:24 remstats Exp $

# - - -   Configuration   - - -

use strict;

# What is this program called, for file-names and error-messages.
$main::prog = 'whois.cgi';
# How to invoke whois
$main::whois = '/usr/bin/whois';
# Where to send them if no server is specified
$main::default_server = 'whois.arin.net';

# Debugging anyone?
$main::debug = 0;

# - - -   Version History   - - -

(undef, $main::version) = split(' ', '$Revision: 1.5 $');

# - - -   Setup   - - -

$ENV{PATH} = '/usr/lib/remstats/bin:/usr/bin:/usr/local/bin:/usr/sbin:/usr/local/sbin';

use lib '.', '/usr/lib/remstats/lib', '/usr/lib/perl5/';
require "cgistuff.pl";

# Initialize some useful variables
%main::R = &cgi_request;
&cgi_sendheaders('Description: remstats whois.cgi');
$main::url = 'http://' . &cgi_var("SERVER_NAME") . ':' . &cgi_var('SERVER_PORT')
	. &cgi_var('SCRIPT_NAME');

# There are lots of NICs now.  The common ones I know for looking up hosts/IP#s are:
my %nics = (
	'whois.ripe.net' => '%s',
	'whois.apnic.net' => '%s',
	'whois.ripe.net' => '%s',
);

&cgi_fmtrequest(%main::R) if ($main::debug);

# Is this really fwhois?  Isn't incompatible syntax wonderful? NFC
if (-l $main::whois) {
	&debug("whois is a symlink") if ($main::debug);
	my $real_whois = readlink $main::whois or
		&abort("error reading whois symlink: $main::whois");
	if ($real_whois =~ m#fwhois#) { $main::fwhois = 1; }
	else { $main::fwhois = 0; }
}
elsif ($main::whois =~ m#fwhois#) { $main::fwhois = 1; }
else { $main::fwhois = 0; }
&debug("fwhois=<B>$main::fwhois</B>") if ($main::debug);

# - - -   Main Line   - - -

# Where are we going to get the info
my ($server, $query);
if (defined $main::R{'server'}) {
	&debug("server=<B>$main::R{'server'}</B>") if ($main::debug);
	if ($main::R{'server'} =~ /^([-a-zA-Z0-9.]+)$/) {
		$server = $1;
	}
	else { &abort("malformed server string '$main::R{'server'}'"); }
}
else { $server = $main::default_server; }

# What are we asking about?
$query = '';
if (defined $main::R{'query'}) {
	&debug("query=<B>$main::R{'query'}</B>") if ($main::debug);
	if ($main::R{'query'} =~ m#^([-a-zA-Z0-9_\./]+)$#) { $query = $1; }
	else { &abort("malformed query string '$main::R{'query'}'"); }
}

$| = 1; # no buffering please
print <<"EOD_FORM";
<HTML>
<HEAD>
<TITLE>$main::prog $query</TITLE>
</HEAD>
<BODY>
<H1>$main::prog (version $main::version) $query</H1>
<FORM METHOD="POST" ACTION="$main::url">
Query:
<INPUT NAME="query" SIZE=20 VALUE="$query">
Server:
<INPUT NAME="server" SIZE=20 VALUE="$server">
<INPUT TYPE="SUBMIT" VALUE="Do it">
</FORM>
EOD_FORM

unless (defined $query) {
	print "</BODY></HTML>\n";
	exit 0;
}

my $cmd = &make_whois( $server, $query);

print "<HR><PRE>\n";

# Collect the info
#my @lines = ();
#open (PIPE, "$cmd|") or &abort("can't open pipe from $cmd");
#@lines = <PIPE>;
#close (PIPE);
my @lines = `$cmd`;

&debug("raw whois info:\n<BR>" . join("\n<BR>", @lines)) if ($main::debug);

# Look for indirection
my $found = 0;
foreach (@lines) {
	next unless (/^\s+Whois\s+Server:\s+(.*)$/i);
	$server = $1;
	&debug("found indirect server <B>$server</B>") if ($main::debug);
	last;
}

# If we found indirection, follow it
if ($found) {
	$cmd = &make_whois( $server, $query);
	@lines = `$cmd`;
	if ($?) { &abort("can't run whois2: $?"); }
	&debug("raw indirect whois info:\n<BR>" . join("\n<BR>", @lines)) if ($main::debug);
}

my ($tag, $addr, $nic, $newquery);
foreach (@lines) {
	if (m#\(([A-Z][-A-Z0-9]+)\)#) {
		$tag = $1;
		unless ($tag =~ /^(ph|phone|fax)$/i) {
			s#\($tag\)#\(<A HREF="$main::url?server=$server&query=$tag">$tag</A>\)#;
		}
	}

	if (m#\s([-a-z0-9_\.]+\@[-a-z0-9\.]+)\s#i) {
		$addr = $1;
		s#$addr#<A HREF="mailto:$addr">$addr</A>#;
	}
	foreach $nic (keys %nics) {
		if (/$nic/i) {
			$newquery = sprintf $nics{$nic}, $query;
			s#${nic}#<A HREF="${main::url}\?server=${nic}\&query=${newquery}">${nic}</A>#i;
			&debug("subbed in link to <B>$nic</B>") if ($main::debug);
		}
	}
	print $_;
}
print <<"EOD_FOOTER";
</PRE>
<HR>
<A HREF="$main::url?server=$server&query=$query">Do it again</A>
</BODY>
</HTML>
EOD_FOOTER

0;

sub abort {
	my ($msg) = @_;
	print <<"EOD_ABORT";
<H1>Abort</H1>
$msg
</BODY>
</HTML>
EOD_ABORT
	exit 1;
}

sub make_whois {
	my ($server, $query) = @_;
# *&%*&$* fwhois
	my $cmd = $main::whois .' ';
	if ($main::fwhois) { $cmd .= "'". $query ."'@". $server; }
	else { $cmd .= ' -h '. $server ." '". $query ."'"; }
	&debug("whois cmd=<B>$cmd</B>") if ($main::debug);
$cmd;
}

sub debug {
	my $msg = join('', @_);
	print "<BR>DEBUG: $msg<BR>\n";
}
